package fr.itsonus.bousoleplussbackend.domain.quiz.model;

import lombok.Data;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Objects;

@Data
public class QuizDetailed {
    private Long id;
    private Date createdDate;
    private List<AxeResponses> axes = new ArrayList<>();

    public static QuizDetailed from(Quiz quiz, List<QuizResponse> responses) {
        var resume = new QuizDetailed();
        resume.setId(quiz.getId());
        resume.setCreatedDate(quiz.getCreatedDate());
        var sortedResponses = new HashMap<Integer, AxeResponses>();
        responses.forEach(quizResponse -> sortedResponses.compute(quizResponse.axeIdentifier(), (k, v) -> {
            var axeResponses = Objects.requireNonNullElseGet(v, () -> AxeResponses.builder().axeIdentifier(k).responses(new ArrayList<>()).build());
            axeResponses.getResponses().add(quizResponse);
            return axeResponses;
        }));
        resume.axes = sortedResponses.values().stream().toList();
        resume.axes.forEach(axeResponses -> {
            var average = axeResponses.getResponses().stream().mapToDouble(QuizResponse::score).average().orElse(0.0);
            axeResponses.setAverage(average);
        });
        return resume;
    }
}
