package fr.itsonus.bousoleplussbackend.infrastructure.postgres.models;

import fr.itsonus.bousoleplussbackend.domain.quiz.model.Bundle;
import jakarta.persistence.Entity;
import jakarta.persistence.GeneratedValue;
import jakarta.persistence.Id;
import jakarta.persistence.JoinColumn;
import jakarta.persistence.ManyToOne;
import jakarta.persistence.OneToMany;
import jakarta.validation.constraints.NotBlank;
import jakarta.validation.constraints.Size;
import lombok.Getter;
import lombok.Setter;
import lombok.ToString;
import org.hibernate.Hibernate;

import java.util.Comparator;
import java.util.Objects;
import java.util.Set;

@Getter
@Setter
@ToString
@Entity(name = "bundle")
public class PostgresBundleDto {

    @Id
    @GeneratedValue
    private Long id;

    @NotBlank
    @Size(max = 50)
    private String label;

    private String presentation;

    @OneToMany(mappedBy = "bundle")
    @ToString.Exclude
    private Set<PostgresQuestionDto> questions;

    @OneToMany(mappedBy = "bundle")
    @ToString.Exclude
    private Set<PostgresQuizDto> quizzes;

    @ManyToOne
    @JoinColumn(name = "user_id")
    private PostgresUserDto user;

    public Bundle toBundle() {
        return Bundle.builder()
                .id(id)
                .label(label)
                .presentation(presentation)
                .lastQuizzDate(quizzes != null ? quizzes.stream().max(Comparator.comparing(PostgresQuizDto::getCreatedDate)).map(PostgresQuizDto::getCreatedDate).orElse(null): null)
                .numberOfQuizzes(quizzes != null ? quizzes.size(): 0)
                .build();
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || Hibernate.getClass(this) != Hibernate.getClass(o)) return false;
        PostgresBundleDto axe = (PostgresBundleDto) o;
        return id != null && Objects.equals(id, axe.id);
    }

    @Override
    public int hashCode() {
        return getClass().hashCode();
    }
}
