package fr.itsonus.bousoleplussbackend.infrastructure.postgres.repositories;

import fr.itsonus.bousoleplussbackend.domain.auth.model.User;
import fr.itsonus.bousoleplussbackend.domain.quiz.model.Bundle;
import fr.itsonus.bousoleplussbackend.domain.quiz.model.Question;
import fr.itsonus.bousoleplussbackend.domain.quiz.spi.BundleCacheRepository;
import fr.itsonus.bousoleplussbackend.infrastructure.postgres.UserCachePostgresRepository;
import fr.itsonus.bousoleplussbackend.infrastructure.postgres.models.PostgresBundleDto;
import fr.itsonus.bousoleplussbackend.infrastructure.postgres.models.PostgresQuestionDto;
import jakarta.transaction.Transactional;
import lombok.AllArgsConstructor;
import org.springframework.data.rest.webmvc.ResourceNotFoundException;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
@AllArgsConstructor
public class BundleCacheProxyRepository implements BundleCacheRepository {

    private final BundleCachePostgresRepository bundleCachePostgresRepository;
    private final QuestionCachePostgresRepository questionCachePostgresRepository;
    private final AxeCachePostgresRepository axeCachePostgresRepository;
    private final UserCachePostgresRepository userCachePostgresRepository;

    public Bundle findByIdAndUserId(Long id, Long userId) {
        return bundleCachePostgresRepository.findByIdAndUserId(id, userId)
                .map(PostgresBundleDto::toBundle)
                .orElseThrow(() -> new ResourceNotFoundException("Bundle not found"));
    }

    @Override
    public List<Bundle> findAllByUser(User user) {
        return bundleCachePostgresRepository.findAllByUserId(user.getId())
                .stream().map(PostgresBundleDto::toBundle)
                .toList();
    }

    @Override
    @Transactional
    public Bundle save(String label, String presentation, List<Question> questions, User user) {
        var entity = new PostgresBundleDto();
        entity.setLabel(label);
        entity.setPresentation(presentation);
        entity.setUser(userCachePostgresRepository.findById(user.getId()).orElseThrow());
        var newEntity = bundleCachePostgresRepository.save(entity);
        questions.forEach(question -> {
            var questionEntity = new PostgresQuestionDto();
            questionEntity.setBundle(newEntity);
            questionEntity.setDescription(question.getDescription());
            questionEntity.setLabel(question.getLabel());
            questionEntity.setAxe(axeCachePostgresRepository.findById(question.getAxeId()));
            questionEntity.setIndex(question.getIndex());
            questionCachePostgresRepository.save(questionEntity);
        });
        return newEntity.toBundle();
    }
}
