package fr.itsonus.bousoleplussbackend.usecase;

import fr.itsonus.bousoleplussbackend.domain.auth.AuthenticationService;
import fr.itsonus.bousoleplussbackend.domain.auth.spi.UserCacheRepository;
import fr.itsonus.bousoleplussbackend.exception.InvalidPasswordException;
import lombok.RequiredArgsConstructor;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

@Service
@RequiredArgsConstructor
public class UserUpdateUseCase {

    private final UserCacheRepository userRepository;
    private final AuthenticationService authenticationService;
    private final PasswordEncoder passwordEncoder;

    public void update(final String username, final String email) {
        var user = authenticationService.getCurrentUser();
        userRepository.save(user.toBuilder().email(email).username(username).build());
    }

    public void updatePassword(final String currentPassword, final String newPassword, final String confirmPassword) {
        var user = authenticationService.getCurrentUser();
        if (!newPassword.equals(confirmPassword)) {
            throw new InvalidPasswordException("Saisir le même mot de passe dans les champs 'Mot de passe' et 'Confirmation du mot de passe'.");
        }
        if (!passwordEncoder.matches(currentPassword, user.getPassword())) {
            throw new InvalidPasswordException("Saisir le bon mot de passe dans le champs 'Mot de passe'");
        }
        userRepository.save(user.toBuilder().password(passwordEncoder.encode(newPassword)).build());
    }
}
