CREATE SEQUENCE IF NOT EXISTS users_seq START WITH 1 INCREMENT BY 50;
CREATE SEQUENCE IF NOT EXISTS refresh_token_seq START WITH 1 INCREMENT BY 50;
CREATE SEQUENCE IF NOT EXISTS axe_seq START WITH 1 INCREMENT BY 50;
CREATE SEQUENCE IF NOT EXISTS bundle_seq START WITH 1 INCREMENT BY 50;
CREATE SEQUENCE IF NOT EXISTS question_seq START WITH 1 INCREMENT BY 50;
CREATE SEQUENCE IF NOT EXISTS response_seq START WITH 1 INCREMENT BY 50;
CREATE SEQUENCE IF NOT EXISTS quiz_seq START WITH 1 INCREMENT BY 50;

CREATE TABLE users
(
    id       BIGINT       NOT NULL,
    email    VARCHAR(100) NOT NULL,
    username VARCHAR(20)  NOT NULL,
    password VARCHAR(120) NOT NULL,
    CONSTRAINT pk_user PRIMARY KEY (id),
    CONSTRAINT uc_user_email UNIQUE (email)
);

CREATE TABLE refresh_token
(
    id          BIGINT       NOT NULL,
    expiry_date TIMESTAMP    NOT NULL,
    token       VARCHAR(500) NOT NULL,
    user_id     BIGINT       NOT NULL,
    CONSTRAINT pk_refresh_token PRIMARY KEY (id)
);

CREATE TABLE axe
(
    id          BIGINT       NOT NULL,
    identifier  INTEGER      NOT NULL,
    short_title VARCHAR(100) NOT NULL,
    title       VARCHAR(200) NOT NULL,
    description VARCHAR(500) NULL,
    color       VARCHAR(7)   NOT NULL,
    CONSTRAINT pk_axe PRIMARY KEY (id),
    CONSTRAINT uc_axe_identifier UNIQUE (identifier, title, short_title)
);

CREATE TABLE bundle
(
    id          BIGINT       NOT NULL,
    user_id     BIGINT       NOT NULL,
    label       VARCHAR(50)  NOT NULL,
    CONSTRAINT bundle_quiz PRIMARY KEY (id)
);

CREATE TABLE question
(
    id          BIGINT       NOT NULL,
    axe_id      BIGINT       NOT NULL,
    bundle_id   BIGINT       NULL,
    label       VARCHAR(200) NOT NULL,
    description VARCHAR(500) NULL,
    index       INT,
    CONSTRAINT question_quiz PRIMARY KEY (id)
);

CREATE TABLE quiz
(
    id           BIGINT    NOT NULL,
    bundle_id    BIGINT    NOT NULL,
    created_date TIMESTAMP NOT NULL,
    CONSTRAINT pk_quiz PRIMARY KEY (id)
);

CREATE TABLE response
(
    id          BIGINT   NOT NULL,
    question_id BIGINT   NOT NULL,
    quiz_id     BIGINT   NOT NULL,
    score       SMALLINT NOT NULL,
    comment     VARCHAR(500),
    CONSTRAINT pk_response PRIMARY KEY (id)
);